package net.h34t.filemure.repository

import app.cash.sqldelight.db.SqlDriver
import app.cash.sqldelight.driver.jdbc.sqlite.JdbcSqliteDriver
import net.h34t.filemure.ExtId
import net.h34t.filemure.core.entity.*
import net.h34t.filemure.core.entity.Tag.Companion.serialize
import net.h34t.filemure.db.Database
import java.io.InputStream
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

class SqliteRepository(url: String) {

    private val sqliteDtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")

    // private val connection: Connection = DriverManager.getConnection(url)
    private val database: Database

    init {
        val driver: SqlDriver = JdbcSqliteDriver("jdbc:sqlite:test.db")
        Database.Schema.create(driver)
        database = Database(driver)
    }

    private fun toLDT(value: String) = LocalDateTime.parse(value, sqliteDtf)

    fun addFileToLimbo(accountId: Long, filename: String, contentType: String?, size: Long, content: InputStream) {
        database.databaseQueries.insertFileIntoLimbo(
            account_id = accountId,
            ext_id = ExtId.generate().value,
            filename = filename,
            content_type = contentType,
            file_size = size,
            content = content.readAllBytes()
        )
    }

    fun getLimboFileCount(accountId: Long, state: State = State.ACTIVE): Long {
        return database.databaseQueries.getLimboFileCount(account_id = accountId, state = state.code.toLong())
            .executeAsOne()
    }

    fun getFilesInLimbo(accountId: Long, state: State = State.ACTIVE): List<FileRef> {
        return database.databaseQueries.getFilesInLimbo(account_id = accountId, state = state.code.toLong())
            .executeAsList()
            .map {
                FileRef(
                    id = it.id,
                    accountId = it.account_id,
                    extId = it.ext_id,
                    documentId = null,
                    filename = it.filename,
                    fileSize = it.file_size,
                    contentType = it.content_type,
                    contentExtracted = it.content_extracted,
                    created = toLDT(it.created),
                    state = State.fromCode(it.state.toInt())
                )
            }
    }

    fun addDocument(
        accountId: Long,
        title: String,
        referenceDate: LocalDateTime,
        tags: List<Tag>,
        description: String,
        fileExtIds: List<ExtId>
    ): ExtId {
        val extId = ExtId.generate()
        database.databaseQueries.transaction {
            database.databaseQueries.addDocument(
                account_id = accountId,
                ext_id = extId.value,
                title = title,
                description = description,
                tags = tags.serialize(),
                reference_date = referenceDate.format(sqliteDtf)
            )

            val documentId = database.databaseQueries.getLastInsertRowId().executeAsOne()

            database.databaseQueries.attachLimboFilesToDocument(
                account_id = accountId,
                document_id = documentId,
                ext_id = fileExtIds.map { it.value }
            )
        }

        return extId;
    }

    fun getDocuments(accountId: Long, state: State = State.ACTIVE): List<Document> {
        return database.databaseQueries.getDocuments(account_id = accountId, state = state.code.toLong())
            .executeAsList()
            .map {
                Document(
                    id = it.id,
                    extId = it.ext_id,
                    title = it.title,
                    description = it.description,
                    tags = Tag.parse(it.tags),
                    created = toLDT(it.created),
                    referenceDate = toLDT(it.reference_date),
                    state = State.fromCode(it.state.toInt()),
                    files = emptyList()
                )
            }
    }

    fun updateDocument(
        accountId: Long,
        id: Long,
        title: String,
        referenceDate: LocalDateTime,
        tags: List<Tag>,
        description: String,
        state: State = State.ACTIVE,
    ) {
        database.databaseQueries.updateDocument(
            id = id,
            title = title,
            reference_date = referenceDate.format(sqliteDtf),
            tags = tags.serialize(),
            description = description,
            state = state.code.toLong(),
            account_id = accountId,
        )
    }

    fun getDocumentByExtId(accountId: Long, extId: ExtId, state: State): Document {
        return database.databaseQueries.getDocumentByExtId(
            account_id = accountId,
            ext_id = extId.value,
            state = state.code.toLong()
        ).executeAsOne().let { d ->
            Document(
                id = d.id,
                extId = d.ext_id,
                title = d.title,
                description = d.description,
                tags = Tag.parse(d.tags),
                created = toLDT(d.created),
                referenceDate = toLDT(d.reference_date),
                state = State.fromCode(d.state.toInt()),
                files = database.databaseQueries.getFilesForDocument(
                    document_id = d.id,
                    account_id = accountId
                ).executeAsList().map { f ->
                    FileRef(
                        id = f.id,
                        extId = f.ext_id,
                        accountId = f.account_id,
                        documentId = f.document_id,
                        filename = f.filename,
                        contentType = f.content_type,
                        contentExtracted = f.content_extracted,
                        fileSize = f.file_size,
                        created = toLDT(f.created),
                        state = State.fromCode(f.state.toInt())
                    )
                }
            )

        }
    }

    fun loadFile(accountId: Long, extId: ExtId): FileContent {
        return database
            .databaseQueries
            .getFile(account_id = accountId, ext_id = extId.value)
            .executeAsOne().let { f ->
                FileContent(
                    id = f.id,
                    extId = f.ext_id,
                    filename = f.filename,
                    contentType = f.content_type,
                    contentExtracted = f.content_extracted,
                    fileSize = f.file_size,
                    content = f.content
                )
            }
    }
}